@@ -22,10 +22,19 @@ logging.getLogger(__name__).addHandler(logging.NullHandler())
 
 def retrigger_self(payload, proj_envar='_GOOGLE_PROJECT', topic_envar='_TRIGGER_TOPIC'):
     '''
-    Dispatches the next iteration of the function.
+    Dispatches the next iteration of a PubSub-triggered Cloud Function.
 
-    :type payload: :py:class:`dict`
-    :param payload: the pubsub payload.
+    .. code:: python
+
+        from bibtutils.gcp.pubsub import process_trigger, retrigger_self
+        def main(event, context):
+            payload = process_trigger(event, context=context)
+            print(payload)
+            retrigger_self('All work and no play makes Jack a dull boy')
+
+    :type payload: :py:class:`dict` OR :py:class:`str`
+    :param payload: the pubsub payload. can be either a ``dict`` or a ``str``. 
+        will be converted to bytes before sending.
 
     :type proj_envar: :py:class:`str`
     :param proj_envar: (Optional) the environment variable to 
@@ -35,14 +44,9 @@ def retrigger_self(payload, proj_envar='_GOOGLE_PROJECT', topic_envar='_TRIGGER_
     :param topic_envar: (Optional) the environment variable to 
         reference for the triggering pubsub topic. Defaults to ``'_TRIGGER_TOPIC'``.
     '''
-    publisher = pubsub_v1.PublisherClient()
     logging.info(f'Dispatching next worker.')
     topic = f'projects/{os.environ.get(proj_envar)}/topics/{os.environ.get(topic_envar)}'
-    logging.info(f'Payload: {payload}\nPubSub: {topic}')
-    # Convert to Bytes then publish message.
-    payload_bytes = json.dumps(payload).encode('utf-8')
-    publisher.publish(topic=topic, data=payload_bytes)
-    logging.info('PubSub sent.')
+    send_pubsub(topic, payload)
     return
 
 
@@ -51,17 +55,31 @@ def send_pubsub(topic_uri, payload):
     Publishes a pubsub message to the specified topic. Executing account 
     must have pubsub publisher permissions on the topic or in the project.
 
+    .. code:: python
+
+        from bibtutils.gcp.pubsub import process_trigger, send_pubsub
+        def main(event, context):
+            process_trigger(event)
+            topic_uri = f'projects/{os.environ["GOOGLE_PROJECT"]}/topics/{os.environ["NEXT_TOPIC"]}'
+            send_pubsub(
+                topic=topic_uri,
+                payload={'favorite color': 'blue'}
+            )
+
     :type topic_uri: :py:class:`str`
     :param topic_uri: the topic on which to publish. 
         topic uri format: ``'projects/{project_name}/topics/{topic_name}'``
 
-    :type payload: :py:class:`dict`
-    :param payload: the pubsub payload.
+    :type payload: :py:class:`dict` OR :py:class:`str`
+    :param payload: the pubsub payload. can be either a ``dict`` or a ``str``. 
+        will be converted to bytes before sending.
     '''
     publisher = pubsub_v1.PublisherClient()
     logging.info(f'Payload: {payload}\nPubSub: {topic_uri}')
     # Convert to Bytes then publish message.
-    payload_bytes = json.dumps(payload).encode('utf-8')
+    if isinstance(payload, dict):
+        payload = json.dumps(payload)
+    payload_bytes = payload.encode('utf-8')
     publisher.publish(topic=topic_uri, data=payload_bytes)
     logging.info('PubSub sent.')
     return
@@ -70,12 +88,27 @@ def send_pubsub(topic_uri, payload):
 def process_trigger(context, event=None, timeout_secs=1800, 
         fail_alert_webhook_secret_uri='FAIL_ALERT_WEBHOOK_SECRET_URI'):
     '''Check timestamp of triggering event; catches infinite retry 
-    loops on 'retry on fail' cloud functions. If the timeout has been exceeded, 
-    will attempt to alert via Slack after fetching a webhook in Secret Manager 
-    whose name should be provided in the environment variable specified in the 
-    function call. **It expects a full secret URI, not just a name!** If the 
-    triggering pubsub's event is also passed and has a payload, will decode 
-    it as utf-8 and return it. Otherwise, will return ``None``.
+    loops on 'retry on fail' cloud functions. Its good practice to always call
+    this function first in a Cloud Function.
+    
+    If the timeout has been exceeded, will attempt to alert via Slack after 
+    fetching a webhook in Secret Manager whose name should be provided in the 
+    environment variable specified in the function call. **It expects to find a 
+    full secret URI in that environment variable, not just a secret name!**
+    
+    **If (and only if)** the triggering pubsub's event is also passed **and has a payload**, 
+    it will be decoded as utf-8 and returned. Otherwise, this function returns ``None``.
+
+    .. code-block:: python
+    
+        import json
+        from bibtutil.gcp.pubsub import process_trigger
+        def main(event, context):
+            payload = process_trigger(context, event=event)
+            if not payload:
+                raise IOError('No payload in triggering pubsub!')
+            
+            payload = json.loads(payload)
 
     :type context: :class:`google.cloud.functions.Context`
     :param context: the triggering pubsub's context.
@@ -95,6 +128,7 @@ def process_trigger(context, event=None, timeout_secs=1800,
 
     :rtype: :py:class:`str` OR :py:class:`None`
     :returns: the pubsub payload, if present.
+
     '''
     utctime = datetime.now(timezone.utc)
     eventtime = parse(context.timestamp)
